// -*-C++-*-
//
// File:      ProcButton.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Jul 10 10:38:24 MET DST 2001

#include "ProcButton.h"
#include "Xmm.h"
#include "Task.h"
#include <qtoolbar.h>
#include <qtooltip.h>
//#include "proc.xpm"
#include "emproc.xpm"
#include "epproc.xpm"
#include "rgsproc.xpm"
#include "omproc.xpm"

#include "ParamDialog.h"
#include <ExceptionErrorHandler.h>
#include <errstr.h>
#include <qmessagebox.h>

#include <algorithm>

using namespace OdfBrowser;

ProcButton::ProcButton(QToolBar* parent, const char* name) : 
  QToolButton(QPixmap(),QString::null,QString::null,0,0,parent,name),_proc(0)
{

  // << taken from one of the QToolButton constructors
  if ( parent->mainWindow() ) {
    connect( parent->mainWindow(), SIGNAL(pixmapSizeChanged(bool)),
	     this, SLOT(setUsesBigPixmap(bool)) );
    setUsesBigPixmap( parent->mainWindow()->usesBigPixmaps() );
  } else {
    setUsesBigPixmap( false );
  }
  // >>
  
  
  connect(this,SIGNAL(clicked()),this,SLOT(configure()));
  setEnabled(false);  
}

ProcButton::~ProcButton()
{
}

void ProcButton::configure()
{
  
  if (!updateParameters()) return;

  // popup dialog...
  ParameterInfo* info=_proc->parameters();
  bool useTabs = true;
  ParamDialog dialog(info, this, 0, useTabs, "OK");
  dialog.exec();
}

bool ProcButton::updateParameters()
{
  if (!_proc) return false;
  
  bool contents=false;
  string val;
  for (OdfInfo::ObservationExposureList::iterator it=_exposures.begin();
       it != _exposures.end();it++) {
    if (contents) val +=" ";
    val += (*it)->id();
    contents=true;
  }
  
  ParameterInfo* pi=_proc->parameters();
  Parameter* p;
  
  ExceptionErrorHandler local;
  ErrorHandler* oldHandler = errHandler.handler(&local);
  try {
    p=(*pi)["instexpids"];
    p->setNewValue(val);
  
    p=(*pi)["withinstexpids"];
    p->setNewValue(contents?"1":"0"); // true!
  }
  catch (SasException::Error e) {
    QString message(e.message.c_str());
    message += "\nDisabled this proc as of now!";
    QMessageBox::information(this,_proc->name(),message);
    errHandler.handler(oldHandler); // return to default error handler
    _proc=0;
    return false;
  }
  errHandler.handler(oldHandler); // return to default error handler

  
  return true;
}


void ProcButton::selected(OdfInfo::ObservationExposure* e,bool on)
{
  if (!canHandle(e)) return;
  
  // add/remove this exposure:
  OdfInfo::ObservationExposureList::iterator it=
    std::find(_exposures.begin(),_exposures.end(),e);
  if (it != _exposures.end()) {if (!on) _exposures.erase(it);}
  else {if (on) _exposures.push_back(e);}

  // enable/disable this configuration
  on=updateParameters() && !_exposures.empty();
  setEnabled(on);
  emit selected(_proc,on);
}


// --- EMPROC ---
class EmProcButton : public ProcButton
{
public:
  EmProcButton(QToolBar * parent,const char * name=0) : ProcButton(parent,name)
  {
   //
   QToolTip::add( this, "Run 'emproc' on selected exposures" );
   setTextLabel("configure emproc");
   setPixmap(QPixmap(emproc_xpm));
   _proc=task("emproc");
  }
  ~EmProcButton() {}

protected:

  bool canHandle(OdfInfo::ObservationExposure* e)
    {
      return (e->instrument() == Xmm::EMOS1 || e->instrument() == Xmm::EMOS2);
    }
};

// --- RGSPROC ---
class RgsProcButton : public ProcButton
{
public:
  RgsProcButton(QToolBar * parent,const char * name = 0) : 
    ProcButton(parent,name)
    {
      QToolTip::add( this, "Run 'rgsproc' on selected exposures" );
      setTextLabel("configure rgsproc");
      setPixmap(QPixmap(rgsproc_xpm));
      _proc=task("rgsproc");
    }
  ~RgsProcButton() {}

protected:
  bool canHandle(OdfInfo::ObservationExposure* e)
    {
      return (e->instrument() == Xmm::RGS1 || e->instrument() == Xmm::RGS2);
    }
};

// --- EPNPROC ---
class EpnProcButton : public ProcButton
{
public:
  EpnProcButton(QToolBar * parent,const char * name = 0) : 
    ProcButton(parent,name)
    {
      QToolTip::add( this, "Run 'epproc' on selected exposures" );
      setTextLabel("configure epproc");
      setPixmap(QPixmap(epproc_xpm));
      _proc=task("epproc");
    }
  ~EpnProcButton() {}

protected:
  bool canHandle(OdfInfo::ObservationExposure* e)
    {
      return (e->instrument() == Xmm::EPN);
    }
};

// --- OMPROC ---
class OmProcButton : public ProcButton
{
public:
  OmProcButton(QToolBar * parent,const char * name = 0) : 
    ProcButton(parent,name)
    {
      QToolTip::add( this, "Run 'omproc' on selected exposures" );
      setTextLabel("configure omproc");
      setPixmap(QPixmap(omproc_xpm));
      _proc=task("omproc");
    }
  ~OmProcButton() {}

protected:
  bool canHandle(OdfInfo::ObservationExposure* e)
    {
      return (e->instrument() == Xmm::OM);
    }
};

ProcButton* ProcButton::instantiate(Xmm::Instrument i,QToolBar * bar)
{
  switch (i) {
  case Xmm::EMOS1:
  case Xmm::EMOS2:
    return new EmProcButton(bar,"EmProcAction");
  case Xmm::RGS1:
  case Xmm::RGS2:
    return new RgsProcButton(bar,"RgsProcAction");
  case Xmm::EPN:
    return new EpnProcButton(bar,"EpnProcAction");
  case Xmm::OM:
    return new OmProcButton(bar,"OmProcAction");
  default:
    return (ProcButton*)0;
  }
}
